import os
import click


def register(app):
    @app.cli.group()
    def translate():
        """Translation and localization commands."""
        pass

    @translate.command()
    @click.argument('lang')
    def init(lang):
        """Initialize a new language."""
        if os.system('pybabel extract -F babel.cfg -k _l -o messages.pot .'):
            raise RuntimeError('extract command failed')
        if os.system(
                'pybabel init -i messages.pot -d app/translations -l ' + lang):
            raise RuntimeError('init command failed')
        os.remove('messages.pot')

    @translate.command()
    def update():
        """Update all languages."""
        if os.system('pybabel extract -F babel.cfg -k _l -o messages.pot .'):
            raise RuntimeError('extract command failed')
        if os.system('pybabel update -i messages.pot -d app/translations'):
            raise RuntimeError('update command failed')
        os.remove('messages.pot')

    @translate.command()
    def compile():
        """Compile all languages."""
        if os.system('pybabel compile -d app/translations'):
            raise RuntimeError('compile command failed')



from datetime import datetime
from hashlib import md5
from time import time
from flask import current_app
from flask_login import UserMixin
from werkzeug.security import generate_password_hash, check_password_hash
import jwt
from ps_herald import db, login
from ps.Basic import Basic


followers = db.Table(
    'followers',
    db.Column('follower_id', db.Integer, db.ForeignKey('user.id')),
    db.Column('followed_id', db.Integer, db.ForeignKey('user.id'))
)


class User(UserMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(64), index=True, unique=True)
    email = db.Column(db.String(120), index=True, unique=True)
    password_hash = db.Column(db.String(128))
    posts = db.relationship('Post', backref='author', lazy='dynamic')
    about_me = db.Column(db.String(140))
    last_seen = db.Column(db.DateTime, default=datetime.utcnow)
    followed = db.relationship(
        'User', secondary=followers,
        primaryjoin=(followers.c.follower_id == id),
        secondaryjoin=(followers.c.followed_id == id),
        backref=db.backref('followers', lazy='dynamic'), lazy='dynamic')

    def __repr__(self):
        return '<User {}>'.format(self.username)

    def set_password(self, password):
        self.password_hash = generate_password_hash(password)

    def check_password(self, password):
        return check_password_hash(self.password_hash, password)

    def avatar(self, size):
        digest = md5(self.email.lower().encode('utf-8')).hexdigest()
        return 'https://www.gravatar.com/avatar/{}?d=identicon&s={}'.format(
            digest, size)

    def follow(self, user):
        if not self.is_following(user):
            self.followed.append(user)

    def unfollow(self, user):
        if self.is_following(user):
            self.followed.remove(user)

    def is_following(self, user):
        return self.followed.filter(
            followers.c.followed_id == user.id).count() > 0

    def followed_posts(self):
        followed = Post.query.join(
            followers, (followers.c.followed_id == Post.user_id)).filter(
                followers.c.follower_id == self.id)
        own = Post.query.filter_by(user_id=self.id)
        return followed.union(own).order_by(Post.timestamp.desc())

    def get_reset_password_token(self, expires_in=600):
        return jwt.encode(
            {'reset_password': self.id, 'exp': time() + expires_in},
            current_app.config['SECRET_KEY'],
            algorithm='HS256').decode('utf-8')

    @staticmethod
    def verify_reset_password_token(token):
        try:
            id = jwt.decode(token, current_app.config['SECRET_KEY'],
                            algorithms=['HS256'])['reset_password']
        except:
            return
        return User.query.get(id)


@login.user_loader
def load_user(id):
    return User.query.get(int(id))


class Post(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    body = db.Column(db.String(140))
    timestamp = db.Column(db.DateTime, index=True, default=datetime.utcnow)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'))
    language = db.Column(db.String(5))

    def __repr__(self):
        return '<Post {}>'.format(self.body)



class Log(db.Model):
 """The model currently is build around the data structure, the standard python logging mechanism
   uses to send logging Events across the **class logging.StreamHandler()**.
 """
 __tablename__ = 'log'
 __table_args__ = {'sqlite_autoincrement': True} # needed for sqlite. needed for pg ?
 id                = db.Column(db.Integer,primary_key=True)
 created           = db.Column(db.String())     #:e.g. 2015-01-08 10:52:41 currently the time the record is created in the
                                                #bridge as string sqlite does not support datetime per se here, that is, why the
                                                #string representation was choosen.
 threadname        = db.Column(db.String())     #:e.g. MainThread
 api_version       = db.Column(db.String())     #:e.g. 1.2.44
 package_version   = db.Column(db.String())     #:e.g. The version of the module where logging appeared
 name              = db.Column(db.String())     #:e.g. Tests
 relativecreated   = db.Column(db.Float())      #:e.g. 10.5919839592
 process           = db.Column(db.Integer)      #:e.g. 896
 args              = db.Column(db.String())     #:e.g. None
 module            = db.Column(db.String())     #:e.g. t
 funcname          = db.Column(db.String())     #:e.g. run
 levelno           = db.Column(db.Integer)      #:e.g. 40
 processname       = db.Column(db.String())     #:MainProcess
 thread            = db.Column(db.BigInteger)   #:e.g. 140735286908888
 msecs             = db.Column(db.Float())      #:e.g. 616.5919839592
 message           = db.Column(db.String())     #:e.g. "The msg of a log.info statement"
 exc_text          = db.Column(db.String())     #:e.g. None
 exc_info          = db.Column(db.String())     #:e.g. None
 stack_info        = db.Column(db.String())     #:e.g. None
 pathname          = db.Column(db.String())     #:e.g. /Users/setzt/Haufe/basic_package
 filename          = db.Column(db.String())     #:e.g. Basic.py
 asctime           = db.Column(db.String())     #:e.g. ????
 levelname         = db.Column(db.String())     #:INFO
 lineno            = db.Column(db.Integer)      #:e.g. 42
 # These are "extra fields" not in the standard python logging
 produkt_id        = db.Column(db.String(10))   #:e.g. PRODUKT_ID
 system_id         = db.Column(db.String(10))   #:e.g. SYSTEM_ID
 sub_system_id     = db.Column(db.String(10))   #:e.g. SUB_SYSTEM_ID
 sub_sub_system_id = db.Column(db.String(10))   #:e.g. SUB_SUB_SYSTEM_ID
 user_spec_1       = db.Column(db.String(10))   #:e.g. User_specific_value_one
 user_spec_2       = db.Column(db.String(10))   #:e.g. User_specific_value_two
 summary           = db.Column(db.String(70))   #:e.g. concatenation of the before values






 def __repr__(self):
  d={}
  d['created']             = self.created
  d['package_version']     = self.package_version
  d['module']              = self.module
  d['lineno']              = self.lineno
  d['funcname']            = self.funcname
  try:    d['lower_levelname']     = self.levelname.lower()
  except: d['lower_levelname']     = str(self.levelno)

  d['levelname']           = self.levelname
  d['message']             = self.message
  d['exc_text']            = self.exc_text
  d['stack_info']          = self.stack_info
  d['system_id']           = self.system_id
  d['sub_system_id']       = self.sub_system_id
  d['sub_sub_system_id']   = self.sub_sub_system_id
  d['user_spec_1']         = self.user_spec_1
  d['user_spec_2']         = self.user_spec_2
  d['produkt_id']          = self.produkt_id

  return '<tr class="%(lower_levelname)s"> \
                           <td>%(created)s</td> \
                           <td>%(levelname)s</td> \
                           <td>%(system_id)s<br>%(sub_system_id)s<br>%(sub_sub_system_id)s</td> \
                           <td>%(produkt_id)s<br>%(user_spec_1)s<br>%(user_spec_2)s</td>\
                           <td>%(module)s %(package_version)s<br>%(funcname)s<br>line: %(lineno)s<br></td> \
                           <td>%(message)s</td> \
                           <td>%(exc_text)s</td> \
                           <td>%(stack_info)s</td> \
                           </tr>'%d




class HeartBeat(db.Model):
 """Store the HeartBeat of the different systems """
 __tablename__ = 'heartbeat'
 __table_args__ = {'sqlite_autoincrement': True} # needed for sqlite. needed for pg ?

 id                = db.Column(db.Integer,primary_key=True)
 newest_heartbeat  = db.Column(db.String())     #:e.g. 2015-01-08 10:52:41
 system_id         = db.Column(db.String(10),unique=True)   #:e.g. SYSTEM_ID

 def __repr__(self):
     return '<system_id: %r <heartbeat: %r' % (self.system_id, self.newest_heartbeat)









from flask import render_template, current_app
from flask_babel import _
from ps_herald.my_email import send_email


def send_password_reset_email(user):
    token = user.get_reset_password_token()
    send_email(_('[Microblog] Reset Your Password'),
               sender=current_app.config['ADMINS'][0],
               recipients=[user.email],
               text_body=render_template('email/reset_password.txt',
                                         user=user, token=token),
               html_body=render_template('email/reset_password.html',
                                         user=user, token=token))



from flask import render_template
from ps_herald import db
from ps_herald.errors import bp


@bp.app_errorhandler(404)
def not_found_error(error):
    return render_template('errors/404.html'), 404


@bp.app_errorhandler(500)
def internal_error(error):
    db.session.rollback()
    return render_template('errors/500.html'), 500



#!/usr/bin/env python

from datetime import datetime, timedelta
import unittest
import os,sys
  
base_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)),'..')
sys.path.append(base_dir)

from ps_herald import create_app, db
from ps_herald.models import User, Post,Log
from ps_herald import Config


class TestConfig(Config):
    TESTING = True
    SQLALCHEMY_DATABASE_URI = 'sqlite://'
    DEV_STAGE="TESTING"


class UserModelCase(unittest.TestCase):
    def setUp(self):
        os.environ['DEV_STAGE'] = "TESTING"
        self.app = create_app(TestConfig)
        self.app_context = self.app.app_context()
        self.app_context.push()
        db.create_all()

    def tearDown(self):
        db.session.remove()
        db.drop_all()
        self.app_context.pop()

    def test_password_hashing(self):
        u = User(username='susan')
        u.set_password('cat')
        self.assertFalse(u.check_password('dog'))
        self.assertTrue(u.check_password('cat'))

    def test_avatar(self):
        u = User(username='john', email='john@example.com')
        self.assertEqual(u.avatar(128), ('https://www.gravatar.com/avatar/'
                                         'd4c74594d841139328695756648b6bd6'
                                         '?d=identicon&s=128'))

    def test_follow(self):
        u1 = User(username='john', email='john@example.com')
        u2 = User(username='susan', email='susan@example.com')
        db.session.add(u1)
        db.session.add(u2)
        db.session.commit()
        self.assertEqual(u1.followed.all(), [])
        self.assertEqual(u1.followers.all(), [])

        u1.follow(u2)
        db.session.commit()
        self.assertTrue(u1.is_following(u2))
        self.assertEqual(u1.followed.count(), 1)
        self.assertEqual(u1.followed.first().username, 'susan')
        self.assertEqual(u2.followers.count(), 1)
        self.assertEqual(u2.followers.first().username, 'john')

        u1.unfollow(u2)
        db.session.commit()
        self.assertFalse(u1.is_following(u2))
        self.assertEqual(u1.followed.count(), 0)
        self.assertEqual(u2.followers.count(), 0)

    def test_follow_posts(self):
        # create four users
        u1 = User(username='john', email='john@example.com')
        u2 = User(username='susan', email='susan@example.com')
        u3 = User(username='mary', email='mary@example.com')
        u4 = User(username='david', email='david@example.com')
        db.session.add_all([u1, u2, u3, u4])

        # create four posts
        now = datetime.utcnow()
        p1 = Post(body="post from john", author=u1,
                  timestamp=now + timedelta(seconds=1))
        p2 = Post(body="post from susan", author=u2,
                  timestamp=now + timedelta(seconds=4))
        p3 = Post(body="post from mary", author=u3,
                  timestamp=now + timedelta(seconds=3))
        p4 = Post(body="post from david", author=u4,
                  timestamp=now + timedelta(seconds=2))
        db.session.add_all([p1, p2, p3, p4])
        db.session.commit()

        # setup the followers
        u1.follow(u2)  # john follows susan
        u1.follow(u4)  # john follows david
        u2.follow(u3)  # susan follows mary
        u3.follow(u4)  # mary follows david
        db.session.commit()

        # check the followed posts of each user
        f1 = u1.followed_posts().all()
        f2 = u2.followed_posts().all()
        f3 = u3.followed_posts().all()
        f4 = u4.followed_posts().all()
        self.assertEqual(f1, [p2, p4, p1])
        self.assertEqual(f2, [p2, p3])
        self.assertEqual(f3, [p3, p4])
        self.assertEqual(f4, [p4])


if __name__ == '__main__':
    unittest.main(verbosity=2)



from threading import Thread
from flask import current_app
from flask_mail import Message
#from app import mail


def send_async_email(app, msg):
    with app.app_context():
        mail.send(msg)


def send_email(subject, sender, recipients, text_body, html_body):
    msg = Message(subject, sender=sender, recipients=recipients)
    msg.body = text_body
    msg.html = html_body
    Thread(target=send_async_email,
           args=(current_app._get_current_object(), msg)).start()



from flask import render_template, redirect, url_for, flash, request
from werkzeug.urls import url_parse
from flask_login import login_user, logout_user, current_user
#from flask_babel import _
from ps_herald import db
from ps_herald.auth import bp
from ps_herald.auth.forms import LoginForm, RegistrationForm, \
    ResetPasswordRequestForm, ResetPasswordForm
from ps_herald.models import User
from ps_herald.auth.email import send_password_reset_email


@bp.route('/login', methods=['GET', 'POST'])
def login():
    #print("tara")
    if current_user.is_authenticated:
        return redirect(url_for('main.index'))
    form = LoginForm()
    if form.validate_on_submit():
        user = User.query.filter_by(username=form.username.data).first()
        if user is None or not user.check_password(form.password.data):
            flash(('Invalid username or password'))
            return redirect(url_for('auth.login'))
        login_user(user, remember=form.remember_me.data)
        next_page = request.args.get('next')
        if not next_page or url_parse(next_page).netloc != '':
            next_page = url_for('main.index')
        return redirect(next_page)
    #print("weiter mit loginform")

    return render_template('auth/login.html', title=('Sign In'), form=form)


@bp.route('/logout')
def logout():
    logout_user()
    return redirect(url_for('main.index'))


@bp.route('/register', methods=['GET', 'POST'])
def register():
    if current_user.is_authenticated:
        return redirect(url_for('main.index'))
    form = RegistrationForm()
    if form.validate_on_submit():
        user = User(username=form.username.data, email=form.email.data)
        user.set_password(form.password.data)
        db.session.add(user)
        db.session.commit()
        flash(('Congratulations, you are now a registered user!'))
        return redirect(url_for('auth.login'))
    return render_template('auth/register.html', title=('Register'),
                           form=form)


@bp.route('/reset_password_request', methods=['GET', 'POST'])
def reset_password_request():
    if current_user.is_authenticated:
        return redirect(url_for('main.index'))
    form = ResetPasswordRequestForm()
    if form.validate_on_submit():
        user = User.query.filter_by(email=form.email.data).first()
        if user:
            send_password_reset_email(user)
        flash(
            ('Check your email for the instructions to reset your password'))
        return redirect(url_for('auth.login'))
    return render_template('auth/reset_password_request.html',
                           title=('Reset Password'), form=form)


@bp.route('/reset_password/<token>', methods=['GET', 'POST'])
def reset_password(token):
    if current_user.is_authenticated:
        return redirect(url_for('main.index'))
    user = User.verify_reset_password_token(token)
    if not user:
        return redirect(url_for('main.index'))
    form = ResetPasswordForm()
    if form.validate_on_submit():
        user.set_password(form.password.data)
        db.session.commit()
        flash(('Your password has been reset.'))
        return redirect(url_for('auth.login'))
    return render_template('auth/reset_password.html', form=form)



from flask_wtf import FlaskForm
from wtforms import StringField, PasswordField, BooleanField, SubmitField
from wtforms.validators import ValidationError, DataRequired, Email, EqualTo
#from flask_babel import _, lazy_gettext as _l
from ps_herald.models import User


class LoginForm(FlaskForm):
    username = StringField(('Username'), validators=[DataRequired()])
    password = PasswordField(('Password'), validators=[DataRequired()])
    remember_me = BooleanField(('Remember Me'))
    submit = SubmitField(('Sign In'))


class RegistrationForm(FlaskForm):
    username = StringField(('Username'), validators=[DataRequired()])
    email = StringField(('Email'), validators=[DataRequired(), Email()])
    password = PasswordField(('Password'), validators=[DataRequired()])
    password2 = PasswordField(
        ('Repeat Password'), validators=[DataRequired(),
                                           EqualTo('password')])
    submit = SubmitField(('Register'))

    def validate_username(self, username):
        user = User.query.filter_by(username=username.data).first()
        if user is not None:
            raise ValidationError('Please use a different username.')

    def validate_email(self, email):
        user = User.query.filter_by(email=email.data).first()
        if user is not None:
            raise ValidationError('Please use a different email address.')


class ResetPasswordRequestForm(FlaskForm):
    email = StringField(('Email'), validators=[DataRequired(), Email()])
    submit = SubmitField(('Request Password Reset'))


class ResetPasswordForm(FlaskForm):
    password = PasswordField(('Password'), validators=[DataRequired()])
    password2 = PasswordField(
        ('Repeat Password'), validators=[DataRequired(),
                                           EqualTo('password')])
    submit = SubmitField(('Request Password Reset'))



import sys,os
l = os.path.split(os.path.dirname(os.path.abspath(__file__)))[:-1]
#print(l[0])
sys.path.append(l[0])

from ps_herald import create_app, db, cli
from ps_herald.models import User, Post,Log

app = create_app()
cli.register(app)
app.run()
@app.shell_context_processor
def make_shell_context():
    return {'db': db, 'User': User, 'Post': Post}



__doc__="""

  Neelix scans the log and triggers actions.

  It's work is based on two models:
     - Log
     - HeartBeat
 
  First it  analyses the Log-model, which Log.system-id's are available.
  For each system_id, the log is analyzed, if "reactions" should be triggered.
 
  Reactions   are defined in the configuration File. 

  Each system_id has it's own configuration section.

  Within each section currently two values could be defined:

       - MSG
       - AGE

  If either of these values is defined a parameter "MAIL_TO"
  has to be defined. Tho these addresses (a comma seperated list of 
  email recipients Emails will be sent - alerting the error.
  e.g::
 
    [ch2eu]
       MAIL_TO=hedwig@the_othercompany.ch,gottfried@haufe-lexware.com
       MSG="If this text would be found in a logging message would be send."
       #AGE Value is given in seconds
       AGE=60*60*2
    [hu2hu]
       MAIL_TO=hedwig@the_othercompany.ch,gottfried@haufe-lexware.com
       #AGE Value is given in seconds
       AGE=60*60*2
  
  Would consider the only logging-messages from the system-is's ch2eu and hu2hu. No other system_id
  will  be analyzed. Reactions will be triggered, if:

         - either we did not see a log message for the last 2 hours in the two systems
         - or in the ch2eu a logging msg appeared having a text like MSG "


  Usage:
        neelix.py [-v] [-r] [-t]

  Options:

    -h --help     Show this screen.
    -r            reinit model 
    -t            test-only mode 
    -v            verbose

"""

import docopt
import datetime, time
import time
import os
import traceback
import sys
import logging
import pdb
from socket import gethostname

from ps_herald import db,create_app,cli
from ps_herald.models import User, Post, Log,HeartBeat

import ps
from ps.Basic import Basic, send_a_mail
from sqlalchemy import func

class MyConfig(object):
    version = "TBD"

herald_package_version=MyConfig()

ps_basic_singleton = ps.Basic.Basic.get_instance("neelix", guarded_by_lockfile=True)

app = create_app()


TEXT_OF_THE_NOTIFICATION_MESSAGE="""
Dear Madam/Sir, <br>
<br>
 Neelix found a misbehaviour while analyzing the log on %(hostname)s
 The misbehaviour is  on system_id [%(system_id_p)s].
<br>

 The cause of the trigger was: [%(cause_p)s], the firing rule was [%(value_p)s]. 
<br>

 This event will be inserted into the logging messages of system [%(system_id_p)s]
 as an ERROR Event - by neelix.
<br>

 Please consult the Deployment Monitor for further information. 
<br>

 Regards
<br>
  Neelix

<br>
 
In case of questions ... contact  _EP-Produktionsstrecke .
<br>
"""

VERBOSE=False
TEST_ONLY=False


def react(system_id_p, cause_p, value_p, record_p):
   """
      - system_id_p  Name of the system for which we react
      - cause_p      Name of the cause e.g. PATTERN or AGE
      - value_p      Value of the cause                         
      - record_p     table-row of the logging table correlated to the event 

      >>> from ps import Basic
   """
   
   global  VERBOSE
   logger                  = Basic.logger
   hostname = gethostname()
   logger.debug("Neelix reacts sys_id %s: cause %s value %s "%(system_id_p,cause_p,value_p), 
                                  extra={"package_version":herald_package_version.version})
   mail_to=""
   for name,value in Basic.config_parser.items(system_id_p):
      if name=="mail_to": mail_to=value.split(",")
   if mail_to=="":
            mail_to=Basic.l_admin_mail
   message=record_p.message
   send_a_mail("neelix@haufe-lexware.com",\
                mail_to,\
                "neelix %s %s"%(system_id_p,cause_p),\
                TEXT_OF_THE_NOTIFICATION_MESSAGE%(locals()))
   record_p.message     = "Neelix React added Notification %s for %s"%(cause_p, system_id_p)
   record_p.system_id   = system_id_p
   record_p.exc_text    = record_p.message
   record_p.levelno     = 30
   record_p.levelname   = "ERROR"
   record_p.created     = time.strftime("%Y-%m-%d %H:%M:%S")
   di={}
   try:
       for key in record_p.__dict__.keys():
         # skip internal variables and primary key of the table
         if key.startswith("_"): continue 
         if key == "id"        : continue 
         di[key] = record_p.__dict__[key]
       logger.debug("react added new log entry %s "%(str(di)), extra={"package_version":herald_package_version.version})
       row=Log(**di)
       db.session.add(row)
       db.session.commit()
   except:
       print("Start exception ==================================================")
       traceback.print_exc(file=sys.stdout)
       logger.exception("Exception in react", extra={"package_version":herald_package_version.version})
       print("End exception (will rollback transaction now)  ===================")
       db.session.rollback()


def notify(system_id_p,starting_at_p,till_p):
  """
  """
  global  VERBOSE, TEST_ONLY
  if not system_id_p in Basic.config_parser.sections(): return 
  logger                  = Basic.logger
  msg_pattern="Not defined yet"
  allowed_age=9999999999999999999999999
  logger.debug("notify: check %s from %s to %s "%(system_id_p, starting_at_p,till_p), 
                           extra={"package_version":herald_package_version.version})
  for name,value in Basic.config_parser.items(system_id_p):
      #logger.debug("For %s configured : name %s value %s "%(system_id_p, name,value))
      if name=="msg": msg_pattern=value
      if name=="age": allowed_age=eval(value)

  
  #print "notify called with ", system_id_p, "  ", starting_at_p , " pattern ", msg_pattern, " allowed_age ", allowed_age
  #print "notify called with ", system_id_p, "  ", starting_at_p , " pattern ", msg_pattern
  with app.app_context():
      try:
            #ctx = herald_app.test_request_context()
            #ctx.push()
            records = Log.query.filter( Log.created    >=  starting_at_p,
                                               Log.created    <   till_p,
                                               Log.system_id  == system_id_p  ).all()

            for record in records:
              if msg_pattern in str(record.message): 
                   logger.debug("For %s found a PATTERN Log Entry %s "%(str(record)), 
                             extra={"package_version":herald_package_version.version})
                   react(system_id_p,"PATTERN FOUND","msg_pattern", record)
        
            newest = Log.query.filter(Log.system_id  == system_id_p).order_by(Log.created.desc()).first()
            try:
              time_created = time.strptime(newest.created,"%Y-%m-%d %H:%M:%S.%f")
              seconds_newest=time.mktime(time.strptime(newest.created,"%Y-%m-%d %H:%M:%S.%f"))
              seconds_now=time.mktime(time.strptime(str(till_p),"%Y-%m-%d %H:%M:%S.%f"))
            except:
              time_created = time.strptime(newest.created,"%Y-%m-%d %H:%M:%S")
              seconds_newest=time.mktime(time.strptime(newest.created,"%Y-%m-%d %H:%M:%S"))
              seconds_now=time.mktime(time.strptime(str(till_p),"%Y-%m-%d %H:%M:%S.%f"))

            
            real_age = seconds_now - seconds_newest
            logger.debug("For %s the last log message is aged %s seconds: allowed are %s \
                            seconds"%(system_id_p, real_age, allowed_age), 
                        extra={"package_version":herald_package_version.version})
        
            if allowed_age < (seconds_now - seconds_newest):
                   logger.debug("For %s found an AGE Log Entry %s "%(system_id_p, str(newest)), 
                                      extra={"package_version":herald_package_version.version})
                   react(system_id_p,"SYSTEM HEARTBEAT FAILURE", "age", newest)
            if TEST_ONLY:
                 if system_id_p=="neelix": react(system_id_p,"HURZ", "test", newest)

      except:
            print("Start exception ==================================================")
            traceback.print_exc(file=sys.stdout)
            logger.exception("Exception in notify", extra={"package_version":herald_package_version.version})
            print("End exception (will rollback transaction now)  ===================")
            db.session.rollback()


def check():
  """ """
  global  VERBOSE
  initial_heart_beat_date = "0000-01-01 14:05:39" 
  NOW                     = datetime.datetime.now()
  logger                  = Basic.logger
  with app.app_context():
      try:
            #ctx = herald_app.test_request_context()
            #ctx.push()
            system_ids = db.session.query(Log.system_id.distinct().label("system_id")).all()
            for system_id in system_ids:
                system_id  = system_id[0] # system_id is a tuple here, we only want the first value
                try:
                  old_heartbeat_entry = HeartBeat.query.filter(HeartBeat.system_id==system_id).all()[0]
                  newest_heartbeat    = old_heartbeat_entry.newest_heartbeat
                except:
                  newest_heartbeat = initial_heart_beat_date

                # Handle the with notify registered functions
                notify(system_id, newest_heartbeat,NOW)


                try:
                   # Delete the old HeartBeat Entry if such one was available
                   if not newest_heartbeat == initial_heart_beat_date:
                      db.session.delete(old_heartbeat_entry)
                      db.session.commit()
                   #Create the NewHearBeat Entry
                   di={}
                   di['newest_heartbeat'] = NOW
                   di['system_id']        = system_id
                   row=HeartBeat(**di)

                   #Commit
                   db.session.add(row)
                   db.session.commit()
                except:
                   print("Start exception ==================================================")
                   traceback.print_exc(file=sys.stdout)
                   print("End exception (will rollback transaction now)  ===================")
                   db.session.rollback()
                   print ("Except update")
                   pass
        
      except:
            print("Start exception ==================================================")
            traceback.print_exc(file=sys.stdout)
            logger.exception("Exception in check", extra={"package_version":herald_package_version.version})
            print("End exception (will rollback transaction now)  ===================")
            db.session.rollback()

def main():
  """ """
  global  VERBOSE, TEST_ONLY
  options = docopt.docopt(__doc__, version=1 )
  import ps
  retval = 0
  #ps_basic_singleton = ps.Basic.Basic.get_instance("neelix", guarded_by_lockfile=True)
  ps.Basic.Basic.logger.debug("started", extra={"package_version":herald_package_version.version})

  try:
    if options["-v"]: VERBOSE    = True
    if options["-t"]: 
                      VERBOSE    = True
                      TEST_ONLY  = True


    if options["-r"]:     db.create_all()
    else:                 check()
  
  except SystemExit as e:
      ps.Basic.Basic.logger.exception("Exit in main: system exit handler",
                                       extra = {"package_version":herald_package_version.version})
      retval = e.code
  except:
     ps.Basic.Basic.logger.exception("Exception in main: outer handler", 
                                     extra = {"package_version":herald_package_version.version})
     retval = 4

  ps_basic_singleton.__exit__(1, 2, 3)
  return(retval)


if __name__ == "__main__": 
    main()



import json
import requests
from flask import current_app
from flask_babel import _


def translate(text, source_language, dest_language):
    if 'MS_TRANSLATOR_KEY' not in current_app.config or \
            not current_app.config['MS_TRANSLATOR_KEY']:
        return _('Error: the translation service is not configured.')
    auth = {
        'Ocp-Apim-Subscription-Key': current_app.config['MS_TRANSLATOR_KEY']}
    r = requests.get('https://api.microsofttranslator.com/v2/Ajax.svc'
                     '/Translate?text={}&from={}&to={}'.format(
                         text, source_language, dest_language),
                     headers=auth)
    if r.status_code != 200:
        return _('Error: the translation service failed.')
    return json.loads(r.content.decode('utf-8-sig'))



doc_rest="""
"""
__doc__="""

 The Bridge module receives messages emited with the Standard Python logging StreamHandler.
 These messages are stored in a database. 
 The logging events lateron can be analyzed e.g. with the herald Server.

 To use the bridge-functionality, it might be needed to establish a tunnel to another machine,
 another bridge is listening e.g:

   ssh -f -oExitOnForwardFailure=yes   -R 9019:localhost:9024 hr2dev@vl-knecht-ci1.haufe-ep.de  -N
Usage:
  bridge.py [-v] [-b] [-p] [-s AGE ] [-r ROUND ]  

Option:
  -h, --help       Show this screen.
  -v, --verbose    verbose
  -b, --bridge     bridge the received logging event additionaly on the bridge-port 
  -s AGE           delete messages older than age seconds while running ( a week are 604800 seconds)
  -r ROUND         check for messages to delete after that amount of inserts
"""


import docopt
import asyncio
import struct
#import herald_package_version 
herald_package_version ="tBD"
from pickle import loads
import pickle
import time
import os
import sys 
import traceback
import socket
import sys
import logging
import datetime 
import requests
import json
import ps
from ps.Basic import Basic

# make sure we can import from app directory
l = os.path.split(os.path.dirname(os.path.abspath(__file__)))[:-1]
print(l[0])
sys.path.append(l[0])
for i in sys.path:print(i)
sys.stdout.flush()
#base_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)),'..')
#sys.path.append(base_dir)
from ps_herald import db,create_app,cli
from ps_herald.models import User, Post, Log

class MyConfig(object):
    version = "TBD"

herald_package_version=MyConfig()


ps_basic_singleton = ps.Basic.Basic.get_instance("bridge", guarded_by_lockfile=True)
sys.stdout.flush()
app = create_app()
#cli.register(app)

sys.stdout.flush()


CLIENTS = {} # asynchronous tasks are "remembered here"

VERBOSE                      = False
BRIDGE_MODE                  = False
AGE                          = False 
ROUND                        = 1024 

def init_database():
    """ The "None" value in the fields is a placeholder for any-value in the web interface. We ensure, that at least one entry is in the database"""

    with app.app_context():
      try:
        log=Log(produkt_id="None",system_id="None",sub_system_id="None",sub_sub_system_id="None",user_spec_1="None", user_spec_2="None",summary="None")
        db.session.add(log)
        db.session.commit()
      except:
         print(traceback.format_exc())
         Basic.logger.exception("error init database")
         sys.exit(1)



def client_connected_handler(client_reader, client_writer):
    """ Start a new asyncio.Task to handle this specific client connection"""
    task = asyncio.Task(handle_client(client_reader, client_writer))
    CLIENTS[task] = (client_reader, client_writer)
 
    def client_done(task):
        """When the tasks that handles the specific client connection is done"""
        del CLIENTS[task]
 
    # Add the client_done callback to be run when the future becomes done
    task.add_done_callback(client_done)

 
@asyncio.coroutine
def handle_client(client_reader, client_writer):
    """ Runs for each client connected. 
        The protocol used is:  first 4 bite define the len of the message. 
        Hence in the first step, the first four byte are read and the the test of the message.
        This "rest" of the message is depickled and stored in the database.
        If the "bridge-mode" is enable, the "byte-buffer" (first 4 bytes plus the rest of the
        message is also sent to another socket - the bridge socket.
    """
    global socke, VERBOSE, BRIDGE_MODE, AGE, ROUND 
    current_round = 0
    while True:
        current_round += 1
        data1 = yield from (client_reader.read(4))
        if not data1: break
        if len(data1) < 4:
           print ("Did not receive First 4 Bytes of message")
           raise
        slen = struct.unpack('>L', data1)[0]
        data2=yield from (client_reader.read(slen))
        if BRIDGE_MODE: print_to_tunnel(data1 +  data2)    
        obj = loads(data2)
        dt = datetime.datetime.now()
        obj['created'] = dt.strftime("%Y-%m-%d %H:%M:%S.%f")   

        lowered_obj={}
        try:
            # While migrating, there may be pakets without the additional info -  
            lowered_obj["summary"]=str(None)
            lowered_obj["produkt_id"]=str(None)
            lowered_obj["system_id"]=str(None)
            lowered_obj["sub_system_id"]=str(None)
            lowered_obj["sub_sub_system_id"]=str(None)
            lowered_obj["user_spec_1"]=str(None)
            lowered_obj["user_spec_2"]=str(None)
            lowered_obj["api_version"]=str(None)
            lowered_obj["package_version"]=str(None)
            for name,value in obj.items():
                 #: obselete if everything switched to ps.basic > 0.4
                 if name == "PID": name = "produkt_id"
                 #:?? sometimes message appears here ?
                 if name == "msg": 
                                name = "message"
                                value = value.replace(u"\u2018", " ").replace(u"\u2019", " ")
                                value=str(value).replace("'"," ").replace('"'," ")
                 if name == "args": value = ""
                 if value is None: lowered_obj[name.lower()] = str(value)
                 else:             lowered_obj[name.lower()] = value
                 if VERBOSE: print (name.lower() ,"    ", value)
            #
            # The summary Attribute of the model eases searching in the database 
            # later.
            lowered_obj["summary"] = str("None") + lowered_obj["produkt_id"]  \
                                 +lowered_obj["system_id"] \
                                 +lowered_obj["sub_system_id"] +lowered_obj["sub_sub_system_id"] \
                                 +lowered_obj["user_spec_1"]   +lowered_obj["user_spec_2"] 
            row=Log(**lowered_obj)
            with app.app_context():
                 db.session.add(row)
                 db.session.commit()
        #except ConnectionError as e:
        #        Basic.logger.exception("Exception: trying to connect to elastic_search server. ", extra={"package_version":herald_package_version.version})
        #        sys.exit(1) 
        except:
            Basic.logger.exception("Error: while writing to database/elasticsearch", extra={"package_version":"XXX"}) #herald_package_version.version})
            print (lowered_obj)
            traceback.print_exc(file=sys.stderr)
            with app.app_context():
                db.session.rollback()
            continue 
        if VERBOSE: print ("Added a row")
        if AGE and 0 == (current_round % ROUND) :
          current_round = 0
          start_date = str(datetime.datetime.now() -  datetime.timedelta(seconds=AGE)) 
          to_del     = Log.query.filter(Log.created < start_date).delete()
          sys.stdout.write("OK: %s rows deleted"%(to_del))
  
    


socke = None

def print_to_tunnel(data_p):
    global socke
    try:
        socke.sendall(data_p) 
    except AttributeError:
       import pdb
       #pdb.set_trace()
       #Basic.logger.error("Error: while writing to bridge-tunnel ", extra={"package_version":herald_package_version.version})
       socke=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
       socke.connect(("",Basic.logging_bridge_port))
       sys.stderr.write("Attribute Error: while writing to bridge-tunnel ")
    except:
       sys.stderr.write("Error: while writing to bridge-tunnel ")
       #Basic.logger.exception("Error: while writing to bridge-tunnel " )
       traceback.print_exc(file=sys.stdout)
       #reset socket descriptor, so that it will be initialized in the 
       #next round
       import pdb
       socke.close()
       socke=None 

def main():
  """ """
  global  VERBOSE, BRIDGE_MODE, AGE, ROUND  
  options = docopt.docopt(__doc__, version=1 )
  #ps.Basic.Basic.logger.debug("started", extra = {"package_version":herald_package_version.version})
  ps.Basic.Basic.logger.debug("started", extra = {"package_version":herald_package_version})
  print(options)

  sys.stdout.flush()
  if options["--verbose"]: VERBOSE                      = True
  if options["--bridge"] : BRIDGE_MODE                  = True
  if options["-s"]       : AGE                          = int(options["-s"])
  if options["-r"]       : ROUND                        = int(options["-r"])


  init_database()
  sys.stdout.flush()
  loop = asyncio.get_event_loop()
  sys.stdout.flush()
  print("will listen on %d"%(Basic.logging_port))
  loop.run_until_complete(asyncio.start_server(client_connected_handler, '0.0.0.0', Basic.logging_port))
  sys.stdout.flush()
  try:
    loop.run_forever()
  finally:
    loop.close()



if __name__ == "__main__":
     main()



import logging,traceback,sys
from logging.handlers import SMTPHandler, RotatingFileHandler
import os
from flask import Flask, request, current_app
from flask_sqlalchemy import SQLAlchemy
#from flask_migrate import Migrate
from flask_login import LoginManager
from flask_mail import Mail
from flask_bootstrap import Bootstrap
from flask_moment import Moment
#from flask_babel import Babel, lazy_gettext as _l
#from config import Config
#from app.models import User,Log
from ps.Basic import Basic

db = SQLAlchemy()
#migrate = Migrate()
login = LoginManager()
login.login_view = 'auth.login'
login.login_message = ('Please log in to access this page.')
mail = Mail()
bootstrap = Bootstrap()
moment = Moment()
#babel = Babel()

from dotenv import load_dotenv
from ps.Basic import Basic



basedir = os.path.abspath(os.path.dirname(__file__))
load_dotenv(os.path.join(basedir, '.env'))


class Config(object):
    SECRET_KEY = os.environ.get('SECRET_KEY') or 'you-will-never-guess'
    #SQLALCHEMY_DATABASE_URI = os.environ.get('DATABASE_URL') or \
    #    'sqlite:///' + os.path.join(basedir, 'app.db')
    #SQLALCHEMY_DATABASE_URI =   Basic.herald_sqlite_filename
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    MAIL_SERVER = os.environ.get('MAIL_SERVER')
    MAIL_PORT = int(os.environ.get('MAIL_PORT') or 25)
    MAIL_USE_TLS = os.environ.get('MAIL_USE_TLS') is not None
    MAIL_USERNAME = os.environ.get('MAIL_USERNAME')
    MAIL_PASSWORD = os.environ.get('MAIL_PASSWORD')
    ADMINS = ['your-email@example.com']
    LANGUAGES = ['en', 'es']
    #MS_TRANSLATOR_KEY = os.environ.get('MS_TRANSLATOR_KEY')
    POSTS_PER_PAGE = 25


def create_app(config_class=Config):
    app = Flask(__name__)
    app.config.from_object(config_class)

    if Basic.herald_sqlite_filename == "not_yet_defined":
       ps_basic_singleton = Basic.get_instance("herald")

    if app.config['TESTING'] == True: print("Init for TESTING with ")
    else:
        app.config['SQLALCHEMY_DATABASE_URI']='sqlite:///' + Basic.herald_sqlite_filename
        print("The databasefile will be %s"%(app.config['SQLALCHEMY_DATABASE_URI']))
    
    len = 0
    if  os.path.isfile(Basic.herald_sqlite_filename) :
        statinfo = os.stat(Basic.herald_sqlite_filename)
        len = statinfo.st_size

    db.init_app(app)
    
     
    if not os.path.isfile(Basic.herald_sqlite_filename) or len < 1000:
        with app.app_context():
            try:
                db.create_all()
                
            except:
                db.session.rollback()
                print(traceback.format_exc())
                Basic.logger.exception("error init database")
                sys.exit(1)
            print("The database file %s was initialized"%(app.config['SQLALCHEMY_DATABASE_URI'] ))
    else:
            statinfo = os.stat(Basic.herald_sqlite_filename)
            print("Reuse The database file %s was initialized len is %d "%(app.config['SQLALCHEMY_DATABASE_URI'],statinfo.st_size ))

    login.init_app(app)
    mail.init_app(app)
    bootstrap.init_app(app)
    moment.init_app(app)
    #babel.init_app(app)

    from ps_herald.errors import bp as errors_bp
    app.register_blueprint(errors_bp)

    from ps_herald.auth import bp as auth_bp
    app.register_blueprint(auth_bp, url_prefix='/auth')

    from ps_herald.main import bp as main_bp
    app.register_blueprint(main_bp)

    if not app.debug and not app.testing:
        if app.config['MAIL_SERVER']:
            auth = None
            if app.config['MAIL_USERNAME'] or app.config['MAIL_PASSWORD']:
                auth = (app.config['MAIL_USERNAME'],
                        app.config['MAIL_PASSWORD'])
            secure = None
            if app.config['MAIL_USE_TLS']:
                secure = ()
            mail_handler = SMTPHandler(
                mailhost=(app.config['MAIL_SERVER'], app.config['MAIL_PORT']),
                fromaddr='no-reply@' + app.config['MAIL_SERVER'],
                toaddrs=app.config['ADMINS'], subject='Microblog Failure',
                credentials=auth, secure=secure)
            mail_handler.setLevel(logging.ERROR)
            app.logger.addHandler(mail_handler)

        #if not os.path.exists('logs'):
        #    os.mkdir('logs')
        #file_handler = RotatingFileHandler('logs/microblog.log',
        #                                   maxBytes=10240, backupCount=10)
        #file_handler.setFormatter(logging.Formatter(
        #    '%(asctime)s %(levelname)s: %(message)s '
        #    '[in %(pathname)s:%(lineno)d]'))
        #file_handler.setLevel(logging.INFO)
        #app.logger.addHandler(file_handler)
#
        #app.logger.setLevel(logging.INFO)
        #app.logger.info('Microblog startup')
        Basic.logger.info("herald app handler started")

    return app


#@babel.localeselector
#def get_locale():
#    return request.accept_languages.best_match(current_app.config['LANGUAGES'])


from ps_herald import models



import datetime
from flask import render_template, flash, redirect, url_for, request, g, \
    jsonify, current_app, session
from flask_login import current_user, login_required
from flask_babel import _, get_locale
from guess_language import guess_language
from ps_herald import db
from ps_herald.main.forms import EditProfileForm, PostForm
from ps_herald.models import User, Post, Log
from ps_herald.translate import translate
from ps_herald.main import bp
from sqlalchemy.sql import or_


import ps
from ps.Basic import Basic
from socket import gethostname
HOSTNAME=gethostname()
Basic("HERALD")






@bp.before_app_request
def before_request():
    if current_user.is_authenticated:
        current_user.last_seen = datetime.datetime.utcnow()
        db.session.commit()
    #g.locale = str(get_locale())


#@bp.route('/', methods=['GET', 'POST'])
#@bp.route('/index', methods=['GET', 'POST'])
#@login_required
#def index():
#    form = PostForm()
#    if form.validate_on_submit():
#        language = guess_language(form.post.data)
#        if language == 'UNKNOWN' or len(language) > 5:
#            language = ''
#        post = Post(body=form.post.data, author=current_user,
#                    language=language)
#        db.session.add(post)
#        db.session.commit()
#        flash(_('Your post is now live!'))
#        return redirect(url_for('main.index'))
#    page = request.args.get('page', 1, type=int)
#    posts = current_user.followed_posts().paginate(
#        page, current_app.config['POSTS_PER_PAGE'], False)
#    next_url = url_for('main.index', page=posts.next_num) \
#        if posts.has_next else None
#    prev_url = url_for('main.index', page=posts.prev_num) \
#        if posts.has_prev else None
#    return render_template('index.html', title=_('Home'), form=form,
#                           posts=posts.items, next_url=next_url,
#                           prev_url=prev_url)


@bp.route('/',methods=['GET', 'POST'])
@bp.route('/index',methods=['GET', 'POST'])
#@login_required #
def index():
    """ Handle the html-request"""
    import pdb

#    pdb.set_trace()

    #                                              hosts in the haufe-net all start
    #                                              with vl_, we do not need a login here
    dev_stage=Basic.dev_stage

    if  not current_user.is_authenticated and not HOSTNAME.startswith("rs") :
        print("NOE")
        return redirect(url_for('auth.login'))
    #Basic.logger.debug("web interface got a request", extra={"package_version":herald_package_version.version})
    Basic.logger.debug("web interface got a request")
    if not 'pattern'      in session:
                             session["pattern"]      = ""
    if not 'starting_at'  in session:
                             session["starting_at"]   = \
                                                      str(datetime.datetime.now() + \
                                                          datetime.timedelta(days=-5)) # Five days before Today
    if not 'notify_level' in session:
                             session["notify_level"] = 30 #30 is error
    if not 'max_rows'     in session:
                             session["max_rows"]     = 1000
    if not 'old_row_first'in session:
                             session["old_row_first"]= 1
    if not 'PRODUKT_ID'        in session:     session["PRODUKT_ID"]        = "None"
    if not 'SYSTEM_ID'         in session:     session["SYSTEM_ID"]         = "None"
    if not 'SUB_SYSTEM_ID'     in session:     session["SUB_SYSTEM_ID"]     = "None"
    if not 'SUB_SUB_SYSTEM_ID' in session:     session["SUB_SUB_SYSTEM_ID"] = "None"
    if not 'USER_SPEC_1'       in session:     session["USER_SPEC_1"]       = "None"
    if not 'USER_SPEC_2'       in session:     session["USER_SPEC_2"]       = "None"

    if request.method == 'POST':
        session["pattern"]           = request.form['pattern'].strip()
        session["starting_at"]       = request.form['starting_at'].strip()
        session["notify_level"]      = request.form['notify_level']
        session["max_rows"]          = request.form['max_rows']
        session["old_row_first"]     = request.form['old_row_first']
        session["PRODUKT_ID"]               = request.form['PRODUKT_ID'].strip()
        session["SYSTEM_ID"]         = request.form['SYSTEM_ID'].strip()
        session["SUB_SYSTEM_ID"]     = request.form['SUB_SYSTEM_ID'].strip()
        session["SUB_SUB_SYSTEM_ID"] = request.form['SUB_SUB_SYSTEM_ID'].strip()
        session["USER_SPEC_1"]       = request.form['USER_SPEC_1'].strip()
        session["USER_SPEC_2"]       = request.form['USER_SPEC_2'].strip()
    pattern           = session["pattern"]
    starting_at       = session["starting_at"]
    notify_level      = session["notify_level"]
    max_rows          = session["max_rows"]
    old_row_first     = session['old_row_first']
    PRODUKT_ID        = session["PRODUKT_ID"]
    SYSTEM_ID         = session["SYSTEM_ID"]
    SUB_SYSTEM_ID     = session["SUB_SYSTEM_ID"]
    SUB_SUB_SYSTEM_ID = session["SUB_SUB_SYSTEM_ID"]
    USER_SPEC_1       = session["USER_SPEC_1"]
    USER_SPEC_2       = session["USER_SPEC_2"]
    if int(old_row_first) == 1:  user_interface_time_order_of_rows="asc"
    else:                        user_interface_time_order_of_rows="desc"

    def isset(param1,param2):
       """jinjas equalto did not work at this point in time. So this wrapper function
          enables the jinja-template do check if a value within a option was selected
          by the user interface.
          If another way is found to set an <option> value im the jinja template,
          this workaround should be eliminated.
       """
       if str(param1)==str(param2):
             return {"value":param1,"selected":True}
       return {"value":param1,"selected":False}

    unset={'selected': False, 'value': 'None'}
    sset ={'selected': True, 'value': 'None'}
    # The kind, how values for the userinterface are choosen is computing intensive - but generic
    # I am no t quite shure if this approach satisfies future performance requirements
    # An easy way to eliminate the queries (on each call) would be to define the
    # values in a config file.
    pid_query         = db.session.query(Log.produkt_id.distinct().label("produkt_id"))
    pids              = [ isset(row.produkt_id,PRODUKT_ID)                              for row in pid_query.all() ]
    sid_query         = db.session.query(Log.system_id.distinct().label("system_id"))
    system_ids        = [ isset(row.system_id,SYSTEM_ID)                  for row in sid_query.all() ]
    sub_sid_query     = db.session.query(Log.sub_system_id.distinct().label("sub_system_id"))
    sub_system_ids    = [ isset(row.sub_system_id,SUB_SYSTEM_ID)          for row in sub_sid_query.all() ]
    sub_sub_sid_query = db.session.query(Log.sub_sub_system_id.distinct().label("sub_sub_system_id"))
    sub_sub_system_ids= [ isset(row.sub_sub_system_id, SUB_SUB_SYSTEM_ID) for row in sub_sub_sid_query.all() ]
    u1_query          = db.session.query(Log.user_spec_1.distinct().label("user_spec_1"))
    user_spec_1_ids   = [ isset(row.user_spec_1,USER_SPEC_1)              for row in u1_query.all() ]
    u2_query          = db.session.query(Log.user_spec_2.distinct().label("user_spec_2"))
    user_spec_2_ids   = [ isset(row.user_spec_2,USER_SPEC_2)              for row in u2_query.all() ]

    #print sid_query.all()
    #print system_ids

    records = Log.query.filter(
                        Log.created  >=  session["starting_at"],
                        Log.levelno  >=  int(session["notify_level"]),
                        Log.module.like("%"+session["pattern"]+"%"),
                        Log.summary.like("%"+session["PRODUKT_ID"]+"%"),
                        Log.summary.like("%"+session["SYSTEM_ID"]+"%"),
                        Log.summary.like("%"+session["SUB_SYSTEM_ID"]+"%"),
                        Log.summary.like("%"+session["SUB_SUB_SYSTEM_ID"]+"%"),
                        Log.summary.like("%"+session["USER_SPEC_1"]+"%"),
                        Log.summary.like("%"+session["USER_SPEC_2"]+"%"),
                        or_(Log.message.like("%"+session["pattern"]+"%"),
                                Log.module.like("%"+session["pattern"]+"%"),
                                Log.funcname.like("%"+session["pattern"]+"%")
                                ),
                      ).order_by("created %s"%(user_interface_time_order_of_rows)).limit(int(session["max_rows"])).all()
    num_records = len(records)
    return render_template("index.html", **locals())



@bp.route('/explore')
@login_required
def explore():
    page = request.args.get('page', 1, type=int)
    posts = Post.query.order_by(Post.timestamp.desc()).paginate(
        page, current_app.config['POSTS_PER_PAGE'], False)
    next_url = url_for('main.explore', page=posts.next_num) \
        if posts.has_next else None
    prev_url = url_for('main.explore', page=posts.prev_num) \
        if posts.has_prev else None
    return render_template('index.html', title=_('Explore'),
                           posts=posts.items, next_url=next_url,
                           prev_url=prev_url)


@bp.route('/user/<username>')
@login_required
def user(username):
    user = User.query.filter_by(username=username).first_or_404()
    page = request.args.get('page', 1, type=int)
    posts = user.posts.order_by(Post.timestamp.desc()).paginate(
        page, current_app.config['POSTS_PER_PAGE'], False)
    next_url = url_for('main.user', username=user.username,
                       page=posts.next_num) if posts.has_next else None
    prev_url = url_for('main.user', username=user.username,
                       page=posts.prev_num) if posts.has_prev else None
    return render_template('user.html', user=user, posts=posts.items,
                           next_url=next_url, prev_url=prev_url)


@bp.route('/edit_profile', methods=['GET', 'POST'])
@login_required
def edit_profile():
    form = EditProfileForm(current_user.username)
    if form.validate_on_submit():
        current_user.username = form.username.data
        current_user.about_me = form.about_me.data
        db.session.commit()
        flash(_('Your changes have been saved.'))
        return redirect(url_for('main.edit_profile'))
    elif request.method == 'GET':
        form.username.data = current_user.username
        form.about_me.data = current_user.about_me
    return render_template('edit_profile.html', title=_('Edit Profile'),
                           form=form)


@bp.route('/follow/<username>')
@login_required
def follow(username):
    user = User.query.filter_by(username=username).first()
    if user is None:
        flash(_('User %(username)s not found.', username=username))
        return redirect(url_for('main.index'))
    if user == current_user:
        flash(_('You cannot follow yourself!'))
        return redirect(url_for('main.user', username=username))
    current_user.follow(user)
    db.session.commit()
    flash(_('You are following %(username)s!', username=username))
    return redirect(url_for('main.user', username=username))


@bp.route('/unfollow/<username>')
@login_required
def unfollow(username):
    user = User.query.filter_by(username=username).first()
    if user is None:
        flash(_('User %(username)s not found.', username=username))
        return redirect(url_for('main.index'))
    if user == current_user:
        flash(_('You cannot unfollow yourself!'))
        return redirect(url_for('main.user', username=username))
    current_user.unfollow(user)
    db.session.commit()
    flash(_('You are not following %(username)s.', username=username))
    return redirect(url_for('main.user', username=username))


@bp.route('/translate', methods=['POST'])
@login_required
def translate_text():
    return jsonify({'text': translate(request.form['text'],
                                      request.form['source_language'],
                                      request.form['dest_language'])})



@bp.route('/reset_database',methods=[ 'GET'])
@login_required
def reset_database():
    start_date = str(datetime.datetime.now() -  datetime.timedelta(days=30)) # 30 days before Today
    to_del = Log.query.filter(Log.created < start_date).delete()
    return "OK: %s rows deleted"%(to_del)

@bp.route('/shutdown')
def server_shutdown():
   shutdown = request.environ.get('werkzeug.server.shutdown')
   if not shutdown: abort(500)
   shutdown()
   return "Shutting down server"






from setuptools import setup, find_packages
import os
import sys
import subprocess

install_requires= open("requirements.txt").read().split()  
VERSION = open('VERSION.txt').read().strip()

here = os.path.abspath(os.path.dirname(__file__))
open(os.path.join(here,'src', 'ps_herald','herald_package_version.py'), 'w').write('version = "%s"' % VERSION)


setup(name='ps_herald',
      version=VERSION,
      description="Monitoring in the PS environment",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join( "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          #'Development Status :: 5 - Stable',
          'Environment :: Console',
          #'Intended Audience :: DevOps',
          'License :: Other/Proprietary License',
          'Operating System :: POSIX :: Linux',
          #'Programming Language :: Python :: 3.6 :: ',
          'Topic :: Software Development :: Build Tools',
          'Topic :: Utilities'
      ],

      keywords='PS',
      author='Setz',
      url='https://bitbucket.org/drsetz/ps.herald',
      author_email='Thomas.Setz@acm.org',
      license='', # need/want 'ZPL'?
      packages=find_packages('src'),
      package_dir={'':'src'},
      include_package_data=True,
      dependency_links=[],
      zip_safe=False,
      install_requires=install_requires,
      entry_points = {
	'console_scripts': [
	'ps_neelix=ps_herald.ps_neelix:main',
	'ps_bridge=ps_herald.ps_bridge:main',
	'ps_herald=ps_herald.ps_herald:main',
	],
     },
      )



from flask import request
from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, TextAreaField
from wtforms.validators import ValidationError, DataRequired, Length
from flask_babel import _, lazy_gettext as _l
from ps_herald.models import User


class EditProfileForm(FlaskForm):
    username = StringField(_l('Username'), validators=[DataRequired()])
    about_me = TextAreaField(_l('About me'),
                             validators=[Length(min=0, max=140)])
    submit = SubmitField(_l('Submit'))

    def __init__(self, original_username, *args, **kwargs):
        super(EditProfileForm, self).__init__(*args, **kwargs)
        self.original_username = original_username

    def validate_username(self, username):
        if username.data != self.original_username:
            user = User.query.filter_by(username=self.username.data).first()
            if user is not None:
                raise ValidationError(_('Please use a different username.'))


class PostForm(FlaskForm):
    post = TextAreaField(_l('Say something'), validators=[DataRequired()])
    submit = SubmitField(_l('Submit'))

